import sys
import numpy as np
from OpenGL import GL as gl
from PySide2.QtWidgets import QOpenGLWidget, QApplication
from PySide2.QtGui import QOpenGLBuffer, QOpenGLShaderProgram
from PySide2.QtGui import QOpenGLShader, QMatrix4x4, QVector3D
from PySide2.QtCore import Qt
 
class OpenGLWidget(QOpenGLWidget):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("PySide2, OpenGL 3.3")
        self.resize(400, 400)
        self.world_height = 100
        self.mvp_matrix = QMatrix4x4()
        self.proj_matrix = QMatrix4x4()
        self.view_matrix = QMatrix4x4()
        self.model_matrix = QMatrix4x4()
        self.mouse_pressed = False
        self.x_mouse = 0
        self.y_mouse = 0
 
    def initializeGL(self):
        gl.glClearColor(0.2, 0.2, 0.2, 1.0)
        vertShaderSrc = """
            #version 330 core
            in vec3 aPosition;
            uniform mat4 uMvpMatrix;
            void main()
            {
                gl_Position = uMvpMatrix * vec4(aPosition, 1.0);
            }
        """
        fragShaderSrc = """
            #version 330 core
            uniform bool uClicked;
            out vec4 fragColor;
            void main()
            {
                vec4 color;
                
                if (uClicked)
                {
                    color = vec4(1.0, 0.0, 0.0, 1.0);
                }
                else
                {
                    color = vec4(0.95, 0.77, 0.5, 1.0);
                }
                
                fragColor = color;
            }
        """
        self.program = QOpenGLShaderProgram(self)
        self.program.addShaderFromSourceCode(QOpenGLShader.Vertex, vertShaderSrc)
        self.program.addShaderFromSourceCode(QOpenGLShader.Fragment, fragShaderSrc)
        self.program.link()
        self.program.bind()
        vertPositions = np.array([
            -0.5, -0.5, 0.0,
            0.5, -0.5, 0.0,
            0.0, 0.5, 0.0], dtype=np.float32)
        self.vertPosBuffer = QOpenGLBuffer()
        self.vertPosBuffer.create()
        self.vertPosBuffer.bind()
        self.vertPosBuffer.allocate(vertPositions, len(vertPositions) * 4)
        self.program.bindAttributeLocation("aPosition", 0)
        self.program.setAttributeBuffer(0, gl.GL_FLOAT, 0, 3)
        self.program.enableAttributeArray(0)
 
        self.program.bind()
        self.u_mvp_matrix_location = self.program.uniformLocation("uMvpMatrix")
        self.u_clicked_location = self.program.uniformLocation("uClicked")
        self.program.setUniformValue(self.u_clicked_location, 0)
        
        self.model_matrix.translate(QVector3D(30, 30, 0))
        self.model_matrix.scale(QVector3D(50, 50, 1))
 
    def paintGL(self):
        gl.glClear(gl.GL_COLOR_BUFFER_BIT)
        self.mvp_matrix = self.proj_matrix * self.view_matrix * self.model_matrix
        self.program.bind()
        self.program.setUniformValue(self.u_mvp_matrix_location, self.mvp_matrix)
        if self.mouse_pressed:
            self.program.bind()
            self.program.setUniformValue(self.u_clicked_location, 1)
            gl.glDrawArrays(gl.GL_TRIANGLES, 0, 3)
            pixels = np.array([0, 0, 0], dtype=np.ubyte)
            gl.glReadPixels(self.x_mouse, self.y_mouse, 1, 1, gl.GL_RGB, gl.GL_UNSIGNED_BYTE, pixels)
            print(pixels)
            r = pixels[0]
            g = pixels[1]
            b = pixels[2]
            if r == 255 and g == 0 and b == 0:
                print("Picked")
            else:
                print("No")
            self.program.bind()
            self.program.setUniformValue(self.u_clicked_location, 0)
            self.mouse_pressed = False
        gl.glDrawArrays(gl.GL_TRIANGLES, 0, 3)
 
    def resizeGL(self, w, h):
        gl.glViewport(0, 0, w, h)
        aspect = w / h
        world_width = aspect * self.world_height
        self.proj_matrix.setToIdentity()
        self.proj_matrix.ortho(0, world_width, 0, self.world_height, -100, 100)
 
    def mousePressEvent(self, event):
        self.x_mouse = event.x()
        self.y_mouse = self.width() - event.y() - 1;
        self.mouse_pressed = True
        self.update()
 
def main():
    QApplication.setAttribute(Qt.AA_UseDesktopOpenGL)
    a = QApplication(sys.argv)
    w = OpenGLWidget()
    w.show()
    sys.exit(a.exec_())
 
if __name__ == "__main__":
    main()
